package org.nkumar.ant.trace.action;

import com.intellij.openapi.actionSystem.DataContext;
import com.intellij.openapi.editor.Editor;
import com.intellij.openapi.editor.actionSystem.EditorAction;
import com.intellij.openapi.editor.actionSystem.EditorActionHandler;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.ui.DialogBuilder;

import org.nkumar.ant.trace.model.DataHolder;
import org.nkumar.ant.trace.model.DataHolderBuilder;
import org.nkumar.ant.trace.ui.AntTracePanel;
import org.nkumar.ant.trace.ui.UiDataHolder;

/**
 * A simple action that loads the content of the build log,
 * build the model and then displays the same in a dialog box.
 */
public final class AntTracerDisplayAction extends EditorAction
{

    public AntTracerDisplayAction()
    {
        super(new AntTracerEditorActionHandler());
    }

    private static final class AntTracerEditorActionHandler extends EditorActionHandler
    {
        public boolean isEnabled(final Editor editor, final DataContext dataContext)
        {
            //TODO true if the file is xml
            return true;
        }

        public void execute(final Editor editor, final DataContext dataContext)
        {
            try
            {
                //get the content
                final String content = editor.getDocument().getText();

                //build the model
                final DataHolder dataHolder = new DataHolderBuilder().createDataHolder(content);

                //build the UI
                final DialogBuilder dialogBuilder = new DialogBuilder((Project) null);
                dialogBuilder.setCenterPanel(new AntTracePanel(new UiDataHolder(dataHolder)));
                dialogBuilder.setTitle("Ant Tracer");
                dialogBuilder.removeAllActions();
                dialogBuilder.addOkAction();
                dialogBuilder.show();
            }
            catch (Exception ignore)
            {
                //log this exception
            }
        }
    }
}
