package org.nkumar.ant.trace.locator;

import java.io.IOException;

/**
 * A bean holding info that is passed outside this package.
 */
public final class ElementContent
{
    private final String elementName;
    private final String fileName;
    private final int startLineNumber;
    private final int endLineNumber;
    private final String[] content;
    private final String[] startingContent;
    private final String[] endingContent;

    ElementContent(final ElementLocation location, final ElementLocatorFactory factory)
            throws IOException
    {
        this.elementName = location.getElementName();
        this.startLineNumber = location.getStartElementStartLocator().getLineNumber();
        this.endLineNumber = (location.isEmptyElement()) ?
                location.getStartElementEndLocator().getLineNumber() :
                location.getEndElementEndLocator().getLineNumber();
        this.fileName = location.getStartElementStartLocator().getSystemId();
        this.content = factory.getFileContent(this.fileName, this.startLineNumber, this.endLineNumber);
        this.startingContent = factory.getFileContent(this.fileName,
                location.getStartElementStartLocator().getLineNumber(),
                location.getStartElementEndLocator().getLineNumber());
        if (location.isEmptyElement())
        {
            endingContent = new String[0];
        }
        else
        {
            endingContent = factory.getFileContent(this.fileName,
                    location.getEndElementStartLocator().getLineNumber(),
                    location.getEndElementEndLocator().getLineNumber());
        }
    }

    /**
     * Returns the lines having the end tag.
     */
    public String[] getEndingContent()
    {
        return endingContent;
    }

    /**
     * Returns the lines having the starting tag.
     */
    public String[] getStartingContent()
    {
        return startingContent;
    }

    /**
     * Returns all the lines from the starting to the end tag.
     */
    public String[] getContent()
    {
        return content;
    }

    public String getElementName()
    {
        return elementName;
    }

    public int getEndLineNumber()
    {
        return endLineNumber;
    }

    public String getFileName()
    {
        return fileName;
    }

    public int getStartLineNumber()
    {
        return startLineNumber;
    }

    public String toString()
    {
        final StringBuffer buffer = new StringBuffer(512);
        buffer.append("Element Name : ").append(elementName).append("\n");
        buffer.append("Element Location : ").append(fileName).append("@")
                .append(startLineNumber).append(",").append(endLineNumber).append("\n");
        if (content != null && content.length > 0)
        {
            buffer.append("Element Content :").append("\n");
            for (int i = 0; i < content.length; i++)
            {
                buffer.append(content[i]).append("\n");
            }
        }
        return buffer.toString();
    }
}
