package org.nkumar.ant.trace.locator;

import org.xml.sax.Locator;

/**
 * A class holding location information for a particular element.
 */
final class ElementLocation
{
    private final String elementName;
    private final Locator startElementStartLocator;
    private final Locator startElementEndLocator;
    private Locator endElementStartLocator;
    private Locator endElementEndLocator;
    private boolean emptyElement = true;

    public ElementLocation(final String elementName, final Locator startElementStartLocator,
            final Locator startElementEndLocator)
    {
        this.elementName = elementName;
        this.startElementStartLocator = startElementStartLocator;
        this.startElementEndLocator = startElementEndLocator;
    }

    public String getElementName()
    {
        return elementName;
    }

    public Locator getEndElementStartLocator()
    {
        return endElementStartLocator;
    }

    public Locator getStartElementStartLocator()
    {
        return startElementStartLocator;
    }

    public Locator getStartElementEndLocator()
    {
        return startElementEndLocator;
    }


    public Locator getEndElementEndLocator()
    {
        return endElementEndLocator;
    }

    public boolean isEmptyElement()
    {
        return emptyElement;
    }

    public void setEndElementLocators(final Locator endElementStartLocator, final Locator endElementEndLocator)
    {
        if (!(endElementStartLocator.getLineNumber() == endElementEndLocator.getLineNumber()
                && endElementStartLocator.getColumnNumber() == endElementEndLocator.getColumnNumber()))
        {
            this.endElementStartLocator = endElementStartLocator;
            this.endElementEndLocator = endElementEndLocator;
            this.emptyElement = false;
        }
    }

    public String toString()
    {
        final StringBuffer buffer = new StringBuffer(256);
        buffer.append(elementName).append("'s start element is at [[").append(startElementStartLocator)
                .append("],[").append(startElementEndLocator).append("]] ");
        if (!emptyElement)
        {
            buffer.append(" and end element is at [[").append(endElementStartLocator)
                    .append("],[").append(endElementEndLocator).append("]]");
        }
        return buffer.toString();
    }
}
