package org.nkumar.ant.trace.locator;

/**
 * A simple value object storing the name of an element and
 * starting location (line number) of the element in the build file.
 */
final class ElementNameStartLocationKey
{
    private final String elementName;
    private final int startLineNumber;

    public ElementNameStartLocationKey(final String elementName, final int start)
    {
        this.elementName = elementName;
        this.startLineNumber = start;
    }

    /**
     * Returns the name of the element.
     * @return the name of the element
     */
    public String getElementName()
    {
        return elementName;
    }

    /**
     * Return the number of the line where the starting tag of this element ENDs.
     * This is how the ant build is coded.
     * @return the number of the line where the starting tag of this element ENDs
     * @see ElementLocation#getStartElementEndLocator()
     */
    public int getStartLineNumber()
    {
        return startLineNumber;
    }

    public boolean equals(final Object o)
    {
        if (this == o)
        {
            return true;
        }
        if (o == null || getClass() != o.getClass())
        {
            return false;
        }

        final ElementNameStartLocationKey that = (ElementNameStartLocationKey) o;

        return startLineNumber == that.startLineNumber && !(elementName != null ?
                !elementName.equals(that.elementName) : that.elementName != null);

    }

    public int hashCode()
    {
        int result;
        result = (elementName != null ? elementName.hashCode() : 0);
        result = 29 * result + startLineNumber;
        return result;
    }

    public String toString()
    {
        return elementName + "@" + startLineNumber;
    }
}
