package org.nkumar.ant.trace.ui;

import com.intellij.openapi.editor.Editor;
import com.intellij.openapi.editor.EditorSettings;
import com.intellij.openapi.editor.markup.HighlighterLayer;
import com.intellij.openapi.editor.markup.MarkupModel;
import com.intellij.openapi.editor.markup.TextAttributes;
import com.intellij.openapi.ui.Splitter;

import javax.swing.JPanel;
import javax.swing.JSplitPane;
import java.awt.BorderLayout;
import java.awt.Color;
import java.lang.reflect.Method;

/**
 * The panel holding all the UI stuff for this plugin.
 * Basically it contains two scrollable panes separated by a splitter.
 * The two panes have non-editable editors added in them.
 * The content of the panes are highlighted based on the sections in the content.
 * Then four area change listeners are registered, two each with either pane.
 * One each for repainting the central splitter in case either of the pane scrolls.
 * One each to concurrently scroll the other pane while one pane is moved.
 */
public final class AntTracePanel extends JPanel
{
    public static final Color MY_GRAY = new Color(220, 220, 220);
    JSplitPane splitPane;

    public AntTracePanel(final UiDataHolder dataHolder)
    {
        setLayout(new BorderLayout());

        final Splitter splitter = new ConnectingSplitter(dataHolder);
        setDefaultEditorSettings(dataHolder.getLeftEditor());
        setDefaultEditorSettings(dataHolder.getRightEditor());
        addHighLighter(dataHolder.getLeftEditor(), dataHolder.getLeftLineIntervals());
        addHighLighter(dataHolder.getRightEditor(), dataHolder.getRightLineIntervals());
        trySettingTheScrollBarToTheLeft(dataHolder.getLeftEditor());
        splitter.setFirstComponent(dataHolder.getLeftEditor().getComponent());
        splitter.setSecondComponent(dataHolder.getRightEditor().getComponent());
        splitter.setDividerWidth(30);
        add(splitter, BorderLayout.CENTER);
        CommonScroller.registerListeners(dataHolder);
    }

    private static void addHighLighter(final Editor editor, final int[] lineIntervals)
    {
        final MarkupModel markupModel = editor.getMarkupModel();
        //even intervals are highlighted
        for (int i = 1; i < lineIntervals.length - 1; i += 2)
        {
            //TODO check the boundary conditions
            //TODO use range highlighter instead of line highlighter
            for (int j = lineIntervals[i]; j < lineIntervals[i + 1]; j++)
            {
                markupModel.addLineHighlighter(j, HighlighterLayer.LAST, HIGHLIGHTER_ATTRIBUTES);
            }
        }
    }

    private static final TextAttributes HIGHLIGHTER_ATTRIBUTES = new TextAttributes();

    static
    {
        HIGHLIGHTER_ATTRIBUTES.setBackgroundColor(MY_GRAY);
    }

    private static void setDefaultEditorSettings(final Editor editor)
    {
        final EditorSettings settings = editor.getSettings();
        settings.setLineMarkerAreaShown(false);
        settings.setLineNumbersShown(false);
        settings.setWheelFontChangeEnabled(false);
        settings.setAdditionalLinesCount(0);
    }

    //there is a method in EditorEx that can switch the scrollbar from the usual right to the left.
    private static void trySettingTheScrollBarToTheLeft(Editor editor)
    {
        try
        {
            Method method = editor.getClass().getMethod("setVerticalScrollbarOrientation", new Class[]{Integer.TYPE});
            method.invoke(editor, new Object[]{new Integer(0)});
        }
        catch (Exception ignore)
        {
            //ignore
        }
    }
}
